"""Utility functions for the gwsim_pop package."""

from __future__ import annotations

from gwsim_pop.utils.log import get_version_information, setup_logger

__all__ = ["get_version_information", "setup_logger"]
