"""Main entry point for the gwsim_pop package."""

from __future__ import annotations

if __name__ == "__main__":
    from gwsim_pop.utils.log import setup_logger

    setup_logger(print_version=True)
